<?php
/**
 * Plugin Name: WooCommerce Coupon Campaigns & Tracking
 * Version: 1.2.18
 * Plugin URI: https://woocommerce.com/products/woocommerce-coupon-campaigns/
 * Description: Provides the ability to group coupons into campaigns - also offers better tracking and reporting of coupons as well as a bulk coupon generation tool.
 * Author: WooCommerce
 * Author URI: https://woocommerce.com/
 * Text Domain: wc_coupon_campaigns
 * Requires Plugins: woocommerce
 * Requires at least: 5.6
 * Tested up to: 6.5
 * WC tested up to: 8.8
 * WC requires at least: 7.0
 * Woo: 506329:0d1018512ffcfcca48a43da05de22647
 *
 * @package woocommerce-coupon-campaigns
 */

require __DIR__ . '/vendor/autoload.php';

use Automattic\WooCommerce\Grow\Tools\CompatChecker\v0_0_1\Checker;
use Automattic\WooCommerce\Utilities\FeaturesUtil;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

// HPOS compatibility declaration.
add_action(
	'before_woocommerce_init',
	function() {
		if ( class_exists( FeaturesUtil::class ) ) {
			FeaturesUtil::declare_compatibility( 'custom_order_tables', plugin_basename( __FILE__ ), true );
		}
	}
);

if ( ! function_exists( 'wc_coupon_campaigns_tracking' ) && ! function_exists( 'wc_coupon_campaigns_tracking_reports' ) ) {

	function wc_coupon_campaigns_tracking() {
		define( 'WC_COUPON_CAMPAIGNS_VERSION', '1.2.18' ); // WRCS: DEFINED_VERSION.

		// Run compatibility checker checks and bail if not compatible.
		if ( ! Checker::instance()->is_compatible( __FILE__, WC_COUPON_CAMPAIGNS_VERSION ) ) {
			return;
		}

		require_once 'includes/class-wc-coupon-campaigns.php';
		require_once 'includes/class-wc-coupon-campaigns-privacy.php';
		require_once 'includes/class-wc-coupon-campaign-filter.php';

		global $wc_coupon_campaigns;
		$wc_coupon_campaigns = new WC_Coupon_Campaigns( __FILE__ );

		// Load the coupon campaign filter instance.
		( new WC_Coupon_Campaign_Filter() )->load();
	}

	add_action( 'plugins_loaded', 'wc_coupon_campaigns_tracking', 0 );
}
